"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.android = exports.iOS = exports.getText = void 0;
const React = __importStar(require("react"));
const react_portal_1 = require("react-portal");
const lodash_1 = require("lodash");
const tableCol_1 = __importDefault(require("../menus/tableCol"));
const tableRow_1 = __importDefault(require("../menus/tableRow"));
const table_1 = __importDefault(require("../menus/table"));
const formatting_1 = __importDefault(require("../menus/formatting"));
const image_1 = __importDefault(require("../menus/image"));
const FloatingToolbar_1 = __importDefault(require("./FloatingToolbar"));
const LinkEditor_1 = __importDefault(require("./LinkEditor"));
const Menu_1 = __importDefault(require("./Menu"));
const isMarkActive_1 = __importDefault(require("../queries/isMarkActive"));
const getMarkRange_1 = __importDefault(require("../queries/getMarkRange"));
const isNodeActive_1 = __importDefault(require("../queries/isNodeActive"));
const getColumnIndex_1 = __importDefault(require("../queries/getColumnIndex"));
const getRowIndex_1 = __importDefault(require("../queries/getRowIndex"));
exports.getText = content => {
    if (!content) {
        return "";
    }
    else if (content.text) {
        return content.text;
    }
    else if (Array.isArray(content)) {
        return content.map(exports.getText).join("");
    }
    else if (typeof content === "object" && content !== null) {
        return exports.getText(content.content);
    }
};
exports.iOS = () => {
    return ([
        "iPad Simulator",
        "iPhone Simulator",
        "iPod Simulator",
        "iPad",
        "iPhone",
        "iPod",
    ].includes(navigator.platform) ||
        (navigator.userAgent.includes("Mac") && "ontouchend" in document));
};
exports.android = () => {
    const ua = navigator.userAgent.toLowerCase();
    return ua.indexOf("android") > -1;
};
function isActive(props) {
    const { view } = props;
    const { selection } = view.state;
    if (!selection)
        return false;
    if (selection.empty)
        return false;
    if (selection.node && selection.node.type.name === "image") {
        return true;
    }
    if (selection.node)
        return false;
    const slice = selection.content();
    const fragment = slice.content;
    const nodes = fragment.content;
    return lodash_1.some(nodes, n => n.content.size);
}
class SelectionToolbar extends React.Component {
    constructor() {
        super(...arguments);
        this.state = {
            formatHidden: !!this.props.onMakeAnswer,
        };
        this.handleOnSelectLink = ({ href, from, to, title, }) => {
            const { view } = this.props;
            const { state, dispatch } = view;
            console.log(`title`, title, href, from, to);
            if (!from && !to) {
                from = state.selection.from;
                to = state.selection.to;
            }
            if (this.props.onClose) {
                this.props.onClose();
                this.props.view.focus();
            }
            const markType = state.schema.marks.link;
            if (from === to) {
                dispatch(view.state.tr
                    .insertText(title, from, to)
                    .addMark(from, to + title.length, state.schema.marks.link.create({ href })));
            }
            else {
                dispatch(state.tr
                    .removeMark(from, to, markType)
                    .addMark(from, to, markType.create({ href })));
            }
        };
    }
    get suggestedLinkTitle() {
        const { state } = this.props.view;
        const selectionText = state.doc.cut(state.selection.from, state.selection.to).textContent;
        return selectionText.trim() || "";
    }
    get href() {
        const { view } = this.props;
        const { state } = view;
        const { selection } = state;
        const range = getMarkRange_1.default(selection.$from, state.schema.marks.link);
        const mark = range && range.mark;
        return mark ? mark.attrs.href : "";
    }
    render() {
        const _a = this.props, { dictionary, isTemplate, onMakeAnswer } = _a, rest = __rest(_a, ["dictionary", "isTemplate", "onMakeAnswer"]);
        const { view } = rest;
        const { state, dispatch } = view;
        const { selection } = state;
        const isCodeSelection = isNodeActive_1.default(state.schema.nodes.code_block)(state);
        if (isCodeSelection) {
            return null;
        }
        const colIndex = getColumnIndex_1.default(state.selection);
        const rowIndex = getRowIndex_1.default(state.selection);
        const isTableSelection = colIndex !== undefined && rowIndex !== undefined;
        const link = isMarkActive_1.default(state.schema.marks.link)(state);
        const range = getMarkRange_1.default(selection.$from, state.schema.marks.link);
        const isImageSelection = selection.node && selection.node.type.name === "image";
        const selectedText = exports.getText(selection.content());
        let items = [];
        if (isTableSelection) {
            items = table_1.default(dictionary);
        }
        else if (colIndex !== undefined) {
            items = tableCol_1.default(state, colIndex, dictionary);
        }
        else if (rowIndex !== undefined) {
            items = tableRow_1.default(state, rowIndex, dictionary);
        }
        else if (isImageSelection) {
            items = image_1.default(state, dictionary);
        }
        else if (onMakeAnswer && this.state.formatHidden) {
            items = [
                {
                    name: "makeanswer",
                    text: "⬇️ Make Answer",
                },
                {
                    name: "separator",
                },
                {
                    name: "format",
                    text: "Format ...",
                },
            ];
        }
        else {
            items = formatting_1.default(state, isTemplate, dictionary);
            if (onMakeAnswer && !this.state.formatHidden) {
                items.unshift({
                    name: "back",
                    text: "<",
                });
            }
        }
        if (!items.length) {
            return null;
        }
        const { LinkFinder } = this.props;
        const MenuEl = (React.createElement(Menu_1.default, Object.assign({ items: items, onToggleFormat: () => this.setState({ formatHidden: !this.state.formatHidden }), onMakeAnswer: onMakeAnswer }, rest)));
        const LinkEditorEl = !this.href && LinkFinder ? (React.createElement(LinkFinder, { callback: this.handleOnSelectLink, newSearchTerm: this.suggestedLinkTitle, onClose: () => {
                try {
                    if (range && range.from && range.to && !this.href) {
                        const markType = state.schema.marks.link;
                        dispatch(state.tr.removeMark((range && range.from) || 0, (range && range.to) || 0, markType));
                    }
                }
                catch (e) {
                    console.log(`error closing`, e);
                }
                this.props.onClose && this.props.onClose();
            } })) : (React.createElement(LinkEditor_1.default, Object.assign({ dictionary: dictionary, mark: (range && range.mark) || undefined, from: (range && range.from) || selection.from, to: (range && range.to) || selection.to, onSelectLink: this.handleOnSelectLink, onRemoveLink: this.props.onClose }, rest)));
        return link && range && !this.href ? (LinkEditorEl) : (React.createElement(react_portal_1.Portal, null,
            React.createElement(FloatingToolbar_1.default, { view: view, active: isActive(this.props) }, link && range ? LinkEditorEl : MenuEl)));
    }
}
exports.default = SelectionToolbar;
//# sourceMappingURL=SelectionToolbar.js.map